package de.hft.stuttgart.citydoctor2.zip;

import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParseException;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParser;
import de.hft.stuttgart.citydoctor2.parser.InvalidGmlFileException;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.io.Serial;
import java.io.Serializable;
import java.util.zip.ZipEntry;

public class CityGmlZipEntry implements Serializable {

    private static final Logger logger = LogManager.getLogger(CityGmlZipEntry.class);
    @Serial
    private static final long serialVersionUID = -5732913269959043262L;

    private final String fileName;
    private transient CityDoctorModel model;
    private CityGmlZipArchive parentArchive;
    private boolean decompressed;
    private long fileSize = -1L;
    private static final long MB = 1024 * 1024L;
    private ZipEntryErrorType errorType = null;

    public static CityGmlZipEntry of(ZipEntry entry,CityGmlZipArchive parentArchive, ParserConfiguration config){
        CityGmlZipEntry ze = CityGmlZipEntry.register(entry, parentArchive);
        ze.loadEntry(config);
        return ze;
    }

    public void loadEntry(ParserConfiguration config){
        if (decompressed){
            return;
        }
        if (errorType != null){
            logger.warn("Tried loading erroneous CityGmlZipEntry");
            return;
        }
        try{
            this.model = CityGmlParser.parseCityGmlZipEntry(this, config);
            this.decompressed = true;
        } catch (CityGmlParseException | InvalidGmlFileException e) {
            logger.error(e);
            this.errorType = ZipEntryErrorType.INVALID_CITY_GML_FILE;
        } catch (IOException e){
            logger.error(e);
            this.errorType = ZipEntryErrorType.IO_ERROR;
        }
    }

    public static CityGmlZipEntry register(ZipEntry entry, CityGmlZipArchive parentArchive){
        CityGmlZipEntry cgzEntry = new CityGmlZipEntry(entry, parentArchive,false);
        try{
            if (!cgzEntry.entrySizeWithinMemoryLimits()) {
                cgzEntry.errorType = ZipEntryErrorType.EXCESSIVE_FILESIZE;
            }
        } catch (IOException e){
            logger.error(e);
            cgzEntry.errorType = ZipEntryErrorType.IO_ERROR;
        }
        return cgzEntry;
    }

    private boolean entrySizeWithinMemoryLimits() throws IOException {
        long memoryLimit = (long) Math.ceil(((double) Runtime.getRuntime().maxMemory() / MB)*0.9);
        if (fileSize != -1L) {
            return memoryLimit > fileSize;
        }
        try (CityGmlZipInputStream cgis = new CityGmlZipInputStream(this)){
            long filesize = cgis.getFileSize();
            if (filesize != -1){
                this.fileSize = filesize;
                return memoryLimit > fileSize;
            } else {
                return false;
            }
        } catch (Exception e) {
            throw new IOException(e);
        }

    }

    protected CityGmlZipEntry(ZipEntry entry,CityGmlZipArchive parentArchive, boolean decompressed) {
        this.fileName = entry.getName();
        this.model = null;
        this.decompressed = decompressed;
        this.parentArchive = parentArchive;
    }

    public void setArchive(CityGmlZipArchive archive){
        parentArchive = archive;
    }

    public CityGmlZipArchive getArchive(){
        return parentArchive;
    }


    public String getFileName() {
        return fileName;
    }

    public ZipEntryErrorType getErrorType() {
        return errorType;
    }

    public CityDoctorModel getModel() {
        return model;
    }

    public long getFileSize() throws IOException {
        if (fileSize == -1L){
            try(CityGmlZipInputStream cgis = new CityGmlZipInputStream(this)){
                fileSize = cgis.getFileSize();
            }
        }
        return fileSize;
    }
}
